/********************************************************************************
** Form generated from reading UI file 'configure_graph.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGURE_GRAPH_H
#define UI_CONFIGURE_GRAPH_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include <toolgraphics.h>

QT_BEGIN_NAMESPACE

class Ui_WidgetConfigureGraph
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *label;
    DataSourceWidget *dataSourceX;
    QLabel *label_2;
    DataSourceWidget *dataSourceY;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout1;
    QLabel *label_5;
    QLabel *label_6;
    QLabel *label_3;
    QCheckBox *checkBoxAutoX;
    QLineEdit *lineEditMinX;
    QLineEdit *lineEditMaxX;
    QLabel *label_4;
    QCheckBox *checkBoxAutoY;
    QLineEdit *lineEditMinY;
    QLineEdit *lineEditMaxY;
    QGroupBox *groupBox_3;
    QHBoxLayout *hboxLayout;
    QCheckBox *checkBoxShowLines;
    QCheckBox *checkBoxShowPoints;

    void setupUi(QWidget *WidgetConfigureGraph)
    {
        if (WidgetConfigureGraph->objectName().isEmpty())
            WidgetConfigureGraph->setObjectName("WidgetConfigureGraph");
        WidgetConfigureGraph->resize(214, 283);
        vboxLayout = new QVBoxLayout(WidgetConfigureGraph);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(WidgetConfigureGraph);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        dataSourceX = new DataSourceWidget(groupBox);
        dataSourceX->setObjectName("dataSourceX");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(dataSourceX->sizePolicy().hasHeightForWidth());
        dataSourceX->setSizePolicy(sizePolicy);

        gridLayout->addWidget(dataSourceX, 0, 1, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        dataSourceY = new DataSourceWidget(groupBox);
        dataSourceY->setObjectName("dataSourceY");
        sizePolicy.setHeightForWidth(dataSourceY->sizePolicy().hasHeightForWidth());
        dataSourceY->setSizePolicy(sizePolicy);

        gridLayout->addWidget(dataSourceY, 1, 1, 1, 1);


        vboxLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(WidgetConfigureGraph);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout1 = new QGridLayout(groupBox_2);
        gridLayout1->setObjectName("gridLayout1");
        label_5 = new QLabel(groupBox_2);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy1);
        label_5->setAlignment(Qt::AlignCenter);

        gridLayout1->addWidget(label_5, 0, 2, 1, 1);

        label_6 = new QLabel(groupBox_2);
        label_6->setObjectName("label_6");
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);
        label_6->setAlignment(Qt::AlignCenter);

        gridLayout1->addWidget(label_6, 0, 3, 1, 1);

        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName("label_3");

        gridLayout1->addWidget(label_3, 1, 0, 1, 1);

        checkBoxAutoX = new QCheckBox(groupBox_2);
        checkBoxAutoX->setObjectName("checkBoxAutoX");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(checkBoxAutoX->sizePolicy().hasHeightForWidth());
        checkBoxAutoX->setSizePolicy(sizePolicy2);

        gridLayout1->addWidget(checkBoxAutoX, 1, 1, 1, 1);

        lineEditMinX = new QLineEdit(groupBox_2);
        lineEditMinX->setObjectName("lineEditMinX");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(lineEditMinX->sizePolicy().hasHeightForWidth());
        lineEditMinX->setSizePolicy(sizePolicy3);
        lineEditMinX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(lineEditMinX, 1, 2, 1, 1);

        lineEditMaxX = new QLineEdit(groupBox_2);
        lineEditMaxX->setObjectName("lineEditMaxX");
        sizePolicy3.setHeightForWidth(lineEditMaxX->sizePolicy().hasHeightForWidth());
        lineEditMaxX->setSizePolicy(sizePolicy3);
        lineEditMaxX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(lineEditMaxX, 1, 3, 1, 1);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName("label_4");

        gridLayout1->addWidget(label_4, 2, 0, 1, 1);

        checkBoxAutoY = new QCheckBox(groupBox_2);
        checkBoxAutoY->setObjectName("checkBoxAutoY");
        sizePolicy2.setHeightForWidth(checkBoxAutoY->sizePolicy().hasHeightForWidth());
        checkBoxAutoY->setSizePolicy(sizePolicy2);

        gridLayout1->addWidget(checkBoxAutoY, 2, 1, 1, 1);

        lineEditMinY = new QLineEdit(groupBox_2);
        lineEditMinY->setObjectName("lineEditMinY");
        sizePolicy3.setHeightForWidth(lineEditMinY->sizePolicy().hasHeightForWidth());
        lineEditMinY->setSizePolicy(sizePolicy3);
        lineEditMinY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(lineEditMinY, 2, 2, 1, 1);

        lineEditMaxY = new QLineEdit(groupBox_2);
        lineEditMaxY->setObjectName("lineEditMaxY");
        sizePolicy3.setHeightForWidth(lineEditMaxY->sizePolicy().hasHeightForWidth());
        lineEditMaxY->setSizePolicy(sizePolicy3);
        lineEditMaxY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(lineEditMaxY, 2, 3, 1, 1);


        vboxLayout->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(WidgetConfigureGraph);
        groupBox_3->setObjectName("groupBox_3");
        hboxLayout = new QHBoxLayout(groupBox_3);
        hboxLayout->setObjectName("hboxLayout");
        checkBoxShowLines = new QCheckBox(groupBox_3);
        checkBoxShowLines->setObjectName("checkBoxShowLines");

        hboxLayout->addWidget(checkBoxShowLines);

        checkBoxShowPoints = new QCheckBox(groupBox_3);
        checkBoxShowPoints->setObjectName("checkBoxShowPoints");

        hboxLayout->addWidget(checkBoxShowPoints);


        vboxLayout->addWidget(groupBox_3);

        QWidget::setTabOrder(dataSourceX, dataSourceY);
        QWidget::setTabOrder(dataSourceY, checkBoxAutoX);
        QWidget::setTabOrder(checkBoxAutoX, lineEditMinX);
        QWidget::setTabOrder(lineEditMinX, lineEditMaxX);
        QWidget::setTabOrder(lineEditMaxX, checkBoxAutoY);
        QWidget::setTabOrder(checkBoxAutoY, lineEditMinY);
        QWidget::setTabOrder(lineEditMinY, lineEditMaxY);

        retranslateUi(WidgetConfigureGraph);
        QObject::connect(checkBoxAutoX, &QCheckBox::toggled, lineEditMinX, &QLineEdit::setDisabled);
        QObject::connect(checkBoxAutoX, &QCheckBox::toggled, lineEditMaxX, &QLineEdit::setDisabled);
        QObject::connect(checkBoxAutoY, &QCheckBox::toggled, lineEditMinY, &QLineEdit::setDisabled);
        QObject::connect(checkBoxAutoY, &QCheckBox::toggled, lineEditMaxY, &QLineEdit::setDisabled);

        QMetaObject::connectSlotsByName(WidgetConfigureGraph);
    } // setupUi

    void retranslateUi(QWidget *WidgetConfigureGraph)
    {
        groupBox->setTitle(tr2i18n("Data Source", nullptr));
        label->setText(tr2i18n("X :", nullptr));
        label_2->setText(tr2i18n("Y :", nullptr));
        groupBox_2->setTitle(tr2i18n("Ranges", nullptr));
        label_5->setText(tr2i18n("Min:", nullptr));
        label_6->setText(tr2i18n("Max:", nullptr));
        label_3->setText(tr2i18n("X :", nullptr));
        checkBoxAutoX->setText(tr2i18n("auto", nullptr));
        lineEditMinX->setText(tr2i18n("0", nullptr));
        lineEditMaxX->setText(tr2i18n("1", nullptr));
        label_4->setText(tr2i18n("Y :", nullptr));
        checkBoxAutoY->setText(tr2i18n("auto", nullptr));
        lineEditMinY->setText(tr2i18n("0", nullptr));
        lineEditMaxY->setText(tr2i18n("1", nullptr));
        groupBox_3->setTitle(tr2i18n("Options", nullptr));
        checkBoxShowLines->setText(tr2i18n("Show lines", nullptr));
        checkBoxShowPoints->setText(tr2i18n("Show points", nullptr));
        (void)WidgetConfigureGraph;
    } // retranslateUi

};

namespace Ui {
    class WidgetConfigureGraph: public Ui_WidgetConfigureGraph {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGURE_GRAPH_H

