/********************************************************************************
** Form generated from reading UI file 'create_gas_particles.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CREATE_GAS_PARTICLES_H
#define UI_CREATE_GAS_PARTICLES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_WidgetCreateGasParticles
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *label_5;
    QLineEdit *lineEditVolume;
    QLabel *labelVolume;
    QLabel *label;
    QLineEdit *lineEditCount;
    QLabel *labelCount;
    QLabel *label_6;
    QLineEdit *lineEditConcentration;
    QLabel *labelConcentration;
    QLabel *label_2;
    QLineEdit *lineEditMass;
    QLabel *labelMass;
    QLabel *label_3;
    QLineEdit *lineEditTemperature;
    QLabel *labelTemperature;
    QLabel *label_4;
    QLineEdit *lineEditMeanVelocity;
    QLabel *labelMeanVelocity;

    void setupUi(QWidget *WidgetCreateGasParticles)
    {
        if (WidgetCreateGasParticles->objectName().isEmpty())
            WidgetCreateGasParticles->setObjectName("WidgetCreateGasParticles");
        WidgetCreateGasParticles->resize(277, 229);
        vboxLayout = new QVBoxLayout(WidgetCreateGasParticles);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(WidgetCreateGasParticles);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        label_5 = new QLabel(groupBox);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 0, 0, 1, 1);

        lineEditVolume = new QLineEdit(groupBox);
        lineEditVolume->setObjectName("lineEditVolume");
        lineEditVolume->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lineEditVolume->setReadOnly(true);

        gridLayout->addWidget(lineEditVolume, 0, 1, 1, 1);

        labelVolume = new QLabel(groupBox);
        labelVolume->setObjectName("labelVolume");

        gridLayout->addWidget(labelVolume, 0, 2, 1, 1);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        lineEditCount = new QLineEdit(groupBox);
        lineEditCount->setObjectName("lineEditCount");
        lineEditCount->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditCount, 1, 1, 1, 1);

        labelCount = new QLabel(groupBox);
        labelCount->setObjectName("labelCount");

        gridLayout->addWidget(labelCount, 1, 2, 1, 1);

        label_6 = new QLabel(groupBox);
        label_6->setObjectName("label_6");

        gridLayout->addWidget(label_6, 2, 0, 1, 1);

        lineEditConcentration = new QLineEdit(groupBox);
        lineEditConcentration->setObjectName("lineEditConcentration");
        lineEditConcentration->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditConcentration, 2, 1, 1, 1);

        labelConcentration = new QLabel(groupBox);
        labelConcentration->setObjectName("labelConcentration");

        gridLayout->addWidget(labelConcentration, 2, 2, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 3, 0, 1, 1);

        lineEditMass = new QLineEdit(groupBox);
        lineEditMass->setObjectName("lineEditMass");
        lineEditMass->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditMass, 3, 1, 1, 1);

        labelMass = new QLabel(groupBox);
        labelMass->setObjectName("labelMass");

        gridLayout->addWidget(labelMass, 3, 2, 1, 1);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 4, 0, 1, 1);

        lineEditTemperature = new QLineEdit(groupBox);
        lineEditTemperature->setObjectName("lineEditTemperature");
        lineEditTemperature->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditTemperature, 4, 1, 1, 1);

        labelTemperature = new QLabel(groupBox);
        labelTemperature->setObjectName("labelTemperature");

        gridLayout->addWidget(labelTemperature, 4, 2, 1, 1);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 5, 0, 1, 1);

        lineEditMeanVelocity = new QLineEdit(groupBox);
        lineEditMeanVelocity->setObjectName("lineEditMeanVelocity");
        lineEditMeanVelocity->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditMeanVelocity, 5, 1, 1, 1);

        labelMeanVelocity = new QLabel(groupBox);
        labelMeanVelocity->setObjectName("labelMeanVelocity");

        gridLayout->addWidget(labelMeanVelocity, 5, 2, 1, 1);


        vboxLayout->addWidget(groupBox);

        QWidget::setTabOrder(lineEditVolume, lineEditCount);
        QWidget::setTabOrder(lineEditCount, lineEditConcentration);
        QWidget::setTabOrder(lineEditConcentration, lineEditMass);
        QWidget::setTabOrder(lineEditMass, lineEditTemperature);
        QWidget::setTabOrder(lineEditTemperature, lineEditMeanVelocity);

        retranslateUi(WidgetCreateGasParticles);

        QMetaObject::connectSlotsByName(WidgetCreateGasParticles);
    } // setupUi

    void retranslateUi(QWidget *WidgetCreateGasParticles)
    {
        groupBox->setTitle(tr2i18n("Options", nullptr));
        label_5->setText(tr2i18n("Area:", nullptr));
#if QT_CONFIG(tooltip)
        lineEditVolume->setToolTip(tr2i18n("The area the gas takes", nullptr));
#endif // QT_CONFIG(tooltip)
        labelVolume->setText(QString());
        label->setText(tr2i18n("Particle count:", nullptr));
#if QT_CONFIG(tooltip)
        lineEditCount->setToolTip(tr2i18n("The number of particles of this gas.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        lineEditCount->setWhatsThis(tr2i18n("The number of particles of this gas.", nullptr));
#endif // QT_CONFIG(whatsthis)
        lineEditCount->setText(tr2i18n("20", nullptr));
        labelCount->setText(QString());
        label_6->setText(tr2i18n("Concentration:", nullptr));
        labelConcentration->setText(QString());
        label_2->setText(tr2i18n("Particle mass:", nullptr));
#if QT_CONFIG(tooltip)
        lineEditMass->setToolTip(tr2i18n("The mass of one particle", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        lineEditMass->setWhatsThis(tr2i18n("The mass of one particle", nullptr));
#endif // QT_CONFIG(whatsthis)
        lineEditMass->setText(tr2i18n("1", nullptr));
        labelMass->setText(QString());
        label_3->setText(tr2i18n("Temperature:", nullptr));
        lineEditTemperature->setText(tr2i18n("1e21", nullptr));
        labelTemperature->setText(QString());
        label_4->setText(tr2i18n("Mean velocity:", nullptr));
        lineEditMeanVelocity->setText(tr2i18n("(0,0)", nullptr));
        labelMeanVelocity->setText(QString());
        (void)WidgetCreateGasParticles;
    } // retranslateUi

};

namespace Ui {
    class WidgetCreateGasParticles: public Ui_WidgetCreateGasParticles {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CREATE_GAS_PARTICLES_H

