/********************************************************************************
** Form generated from reading UI file 'configure_meter.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGURE_METER_H
#define UI_CONFIGURE_METER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include <toolgraphics.h>

QT_BEGIN_NAMESPACE

class Ui_WidgetConfigureMeter
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    DataSourceWidget *dataSource;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout1;
    QLabel *label;
    QLineEdit *lineEditDigits;

    void setupUi(QWidget *WidgetConfigureMeter)
    {
        if (WidgetConfigureMeter->objectName().isEmpty())
            WidgetConfigureMeter->setObjectName("WidgetConfigureMeter");
        WidgetConfigureMeter->resize(157, 130);
        vboxLayout = new QVBoxLayout(WidgetConfigureMeter);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(WidgetConfigureMeter);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        dataSource = new DataSourceWidget(groupBox);
        dataSource->setObjectName("dataSource");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(dataSource->sizePolicy().hasHeightForWidth());
        dataSource->setSizePolicy(sizePolicy);

        gridLayout->addWidget(dataSource, 0, 0, 1, 1);


        vboxLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(WidgetConfigureMeter);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout1 = new QGridLayout(groupBox_2);
        gridLayout1->setObjectName("gridLayout1");
        label = new QLabel(groupBox_2);
        label->setObjectName("label");

        gridLayout1->addWidget(label, 0, 0, 1, 1);

        lineEditDigits = new QLineEdit(groupBox_2);
        lineEditDigits->setObjectName("lineEditDigits");
        lineEditDigits->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(lineEditDigits, 0, 1, 1, 1);


        vboxLayout->addWidget(groupBox_2);


        retranslateUi(WidgetConfigureMeter);

        QMetaObject::connectSlotsByName(WidgetConfigureMeter);
    } // setupUi

    void retranslateUi(QWidget *WidgetConfigureMeter)
    {
        groupBox->setTitle(tr2i18n("Data Source", nullptr));
        groupBox_2->setTitle(tr2i18n("Options", nullptr));
        label->setText(tr2i18n("Number of digits:", nullptr));
        (void)WidgetConfigureMeter;
    } // retranslateUi

};

namespace Ui {
    class WidgetConfigureMeter: public Ui_WidgetConfigureMeter {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGURE_METER_H

