/********************************************************************************
** Form generated from reading UI file 'configure_step_general.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGURE_STEP_GENERAL_H
#define UI_CONFIGURE_STEP_GENERAL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ConfigureStepGeneralWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QSpinBox *kcfg_floatDisplayPrecision;
    QSpacerItem *spacerItem;
    QCheckBox *kcfg_showAxes;
    QCheckBox *kcfg_showCreationTips;
    QCheckBox *kcfg_enableOpenGL;
    QCheckBox *kcfg_wikiExternal;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *ConfigureStepGeneralWidget)
    {
        if (ConfigureStepGeneralWidget->objectName().isEmpty())
            ConfigureStepGeneralWidget->setObjectName("ConfigureStepGeneralWidget");
        ConfigureStepGeneralWidget->resize(270, 272);
        gridLayout = new QGridLayout(ConfigureStepGeneralWidget);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(ConfigureStepGeneralWidget);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        kcfg_floatDisplayPrecision = new QSpinBox(ConfigureStepGeneralWidget);
        kcfg_floatDisplayPrecision->setObjectName("kcfg_floatDisplayPrecision");
        kcfg_floatDisplayPrecision->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        kcfg_floatDisplayPrecision->setMinimum(1);
        kcfg_floatDisplayPrecision->setMaximum(16);
        kcfg_floatDisplayPrecision->setValue(4);

        gridLayout->addWidget(kcfg_floatDisplayPrecision, 0, 1, 1, 1);

        spacerItem = new QSpacerItem(16, 254, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 0, 2, 6, 1);

        kcfg_showAxes = new QCheckBox(ConfigureStepGeneralWidget);
        kcfg_showAxes->setObjectName("kcfg_showAxes");

        gridLayout->addWidget(kcfg_showAxes, 1, 0, 1, 2);

        kcfg_showCreationTips = new QCheckBox(ConfigureStepGeneralWidget);
        kcfg_showCreationTips->setObjectName("kcfg_showCreationTips");

        gridLayout->addWidget(kcfg_showCreationTips, 2, 0, 1, 2);

        kcfg_enableOpenGL = new QCheckBox(ConfigureStepGeneralWidget);
        kcfg_enableOpenGL->setObjectName("kcfg_enableOpenGL");

        gridLayout->addWidget(kcfg_enableOpenGL, 3, 0, 1, 2);

        kcfg_wikiExternal = new QCheckBox(ConfigureStepGeneralWidget);
        kcfg_wikiExternal->setObjectName("kcfg_wikiExternal");

        gridLayout->addWidget(kcfg_wikiExternal, 4, 0, 1, 2);

        spacerItem1 = new QSpacerItem(235, 101, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem1, 5, 0, 1, 2);


        retranslateUi(ConfigureStepGeneralWidget);

        QMetaObject::connectSlotsByName(ConfigureStepGeneralWidget);
    } // setupUi

    void retranslateUi(QWidget *ConfigureStepGeneralWidget)
    {
        label->setText(tr2i18n("Display precision:", nullptr));
        kcfg_showAxes->setText(tr2i18n("Show scene axes", nullptr));
        kcfg_showCreationTips->setText(tr2i18n("Show tips when creating objects", nullptr));
        kcfg_enableOpenGL->setText(tr2i18n("Enable OpenGL (EXPERIMENTAL)", nullptr));
        kcfg_wikiExternal->setText(tr2i18n("Open wikipedia in external browser", nullptr));
        (void)ConfigureStepGeneralWidget;
    } // retranslateUi

};

namespace Ui {
    class ConfigureStepGeneralWidget: public Ui_ConfigureStepGeneralWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGURE_STEP_GENERAL_H

