/********************************************************************************
** Form generated from reading UI file 'create_softbody_items.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CREATE_SOFTBODY_ITEMS_H
#define UI_CREATE_SOFTBODY_ITEMS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_WidgetCreateSoftBodyItems
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *label_4;
    QLineEdit *lineEditPosition;
    QLabel *label_3;
    QLineEdit *lineEditSize;
    QLabel *label_2;
    QLineEdit *lineEditBodyMass;
    QLabel *label;
    QLineEdit *lineEditYoungModulus;
    QLabel *label1;
    QLineEdit *lineEditBodyDamping;
    QLabel *label2;
    QLineEdit *lineEditSplit;

    void setupUi(QWidget *WidgetCreateSoftBodyItems)
    {
        if (WidgetCreateSoftBodyItems->objectName().isEmpty())
            WidgetCreateSoftBodyItems->setObjectName("WidgetCreateSoftBodyItems");
        WidgetCreateSoftBodyItems->resize(227, 175);
        vboxLayout = new QVBoxLayout(WidgetCreateSoftBodyItems);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(WidgetCreateSoftBodyItems);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setVerticalSpacing(0);
        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 0, 0, 1, 1);

        lineEditPosition = new QLineEdit(groupBox);
        lineEditPosition->setObjectName("lineEditPosition");
        lineEditPosition->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditPosition, 0, 1, 1, 1);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        lineEditSize = new QLineEdit(groupBox);
        lineEditSize->setObjectName("lineEditSize");
        lineEditSize->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditSize, 1, 1, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        lineEditBodyMass = new QLineEdit(groupBox);
        lineEditBodyMass->setObjectName("lineEditBodyMass");
        lineEditBodyMass->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditBodyMass, 2, 1, 1, 1);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 3, 0, 1, 1);

        lineEditYoungModulus = new QLineEdit(groupBox);
        lineEditYoungModulus->setObjectName("lineEditYoungModulus");
        lineEditYoungModulus->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditYoungModulus, 3, 1, 1, 1);

        label1 = new QLabel(groupBox);
        label1->setObjectName("label1");

        gridLayout->addWidget(label1, 4, 0, 1, 1);

        lineEditBodyDamping = new QLineEdit(groupBox);
        lineEditBodyDamping->setObjectName("lineEditBodyDamping");
        lineEditBodyDamping->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditBodyDamping, 4, 1, 1, 1);

        label2 = new QLabel(groupBox);
        label2->setObjectName("label2");

        gridLayout->addWidget(label2, 5, 0, 1, 1);

        lineEditSplit = new QLineEdit(groupBox);
        lineEditSplit->setObjectName("lineEditSplit");
        lineEditSplit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lineEditSplit, 5, 1, 1, 1);


        vboxLayout->addWidget(groupBox);


        retranslateUi(WidgetCreateSoftBodyItems);

        QMetaObject::connectSlotsByName(WidgetCreateSoftBodyItems);
    } // setupUi

    void retranslateUi(QWidget *WidgetCreateSoftBodyItems)
    {
        groupBox->setTitle(tr2i18n("Options", nullptr));
        label_4->setText(tr2i18n("Position:", nullptr));
        lineEditPosition->setText(tr2i18n("(0,0)", nullptr));
        label_3->setText(tr2i18n("Body size:", nullptr));
        lineEditSize->setText(tr2i18n("(1,1)", nullptr));
        label_2->setText(tr2i18n("Body mass:", nullptr));
        lineEditBodyMass->setText(tr2i18n("1", nullptr));
        label->setText(tr2i18n("Young's modulus:", nullptr));
        lineEditYoungModulus->setText(tr2i18n("15", nullptr));
        label1->setText(tr2i18n("Body damping:", nullptr));
        lineEditBodyDamping->setText(tr2i18n("0.3", nullptr));
        label2->setText(tr2i18n("Body split counts:", nullptr));
        lineEditSplit->setText(tr2i18n("(4,4)", nullptr));
        (void)WidgetCreateSoftBodyItems;
    } // retranslateUi

};

namespace Ui {
    class WidgetCreateSoftBodyItems: public Ui_WidgetCreateSoftBodyItems {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CREATE_SOFTBODY_ITEMS_H

