/****************************************************************************
** Meta object code from reading C++ file 'toolgraphics.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../step-26.07.90/step/toolgraphics.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'toolgraphics.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31WidgetVertexHandlerGraphicsItemE_t {};
} // unnamed namespace

template <> constexpr inline auto WidgetVertexHandlerGraphicsItem::qt_create_metaobjectdata<qt_meta_tag_ZN31WidgetVertexHandlerGraphicsItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WidgetVertexHandlerGraphicsItem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WidgetVertexHandlerGraphicsItem, qt_meta_tag_ZN31WidgetVertexHandlerGraphicsItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WidgetVertexHandlerGraphicsItem::staticMetaObject = { {
    QMetaObject::SuperData::link<OnHoverHandlerGraphicsItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31WidgetVertexHandlerGraphicsItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31WidgetVertexHandlerGraphicsItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31WidgetVertexHandlerGraphicsItemE_t>.metaTypes,
    nullptr
} };

void WidgetVertexHandlerGraphicsItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WidgetVertexHandlerGraphicsItem *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *WidgetVertexHandlerGraphicsItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WidgetVertexHandlerGraphicsItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31WidgetVertexHandlerGraphicsItemE_t>.strings))
        return static_cast<void*>(this);
    return OnHoverHandlerGraphicsItem::qt_metacast(_clname);
}

int WidgetVertexHandlerGraphicsItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = OnHoverHandlerGraphicsItem::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN18WidgetGraphicsItemE_t {};
} // unnamed namespace

template <> constexpr inline auto WidgetGraphicsItem::qt_create_metaobjectdata<qt_meta_tag_ZN18WidgetGraphicsItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WidgetGraphicsItem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WidgetGraphicsItem, qt_meta_tag_ZN18WidgetGraphicsItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WidgetGraphicsItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18WidgetGraphicsItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18WidgetGraphicsItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18WidgetGraphicsItemE_t>.metaTypes,
    nullptr
} };

void WidgetGraphicsItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WidgetGraphicsItem *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *WidgetGraphicsItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WidgetGraphicsItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18WidgetGraphicsItemE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StepGraphicsItem"))
        return static_cast< StepGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int WidgetGraphicsItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN12NoteTextEditE_t {};
} // unnamed namespace

template <> constexpr inline auto NoteTextEdit::qt_create_metaobjectdata<qt_meta_tag_ZN12NoteTextEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NoteTextEdit"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NoteTextEdit, qt_meta_tag_ZN12NoteTextEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NoteTextEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12NoteTextEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12NoteTextEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12NoteTextEditE_t>.metaTypes,
    nullptr
} };

void NoteTextEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NoteTextEdit *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *NoteTextEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NoteTextEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12NoteTextEditE_t>.strings))
        return static_cast<void*>(this);
    return KTextEdit::qt_metacast(_clname);
}

int NoteTextEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEdit::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN16NoteGraphicsItemE_t {};
} // unnamed namespace

template <> constexpr inline auto NoteGraphicsItem::qt_create_metaobjectdata<qt_meta_tag_ZN16NoteGraphicsItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NoteGraphicsItem",
        "formatColor",
        "",
        "formatBold",
        "checked",
        "formatAlign",
        "QAction*",
        "action",
        "formatFontFamily",
        "family",
        "formatFontSize",
        "size",
        "currentCharFormatChanged",
        "QTextCharFormat",
        "f",
        "cursorPositionChanged",
        "insertImage",
        "insertFormula"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'formatColor'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'formatBold'
        QtMocHelpers::SlotData<void(bool)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'formatAlign'
        QtMocHelpers::SlotData<void(QAction *)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'formatFontFamily'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'formatFontSize'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'currentCharFormatChanged'
        QtMocHelpers::SlotData<void(const QTextCharFormat &)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'cursorPositionChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertImage'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertFormula'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NoteGraphicsItem, qt_meta_tag_ZN16NoteGraphicsItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NoteGraphicsItem::staticMetaObject = { {
    QMetaObject::SuperData::link<WidgetGraphicsItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16NoteGraphicsItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16NoteGraphicsItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16NoteGraphicsItemE_t>.metaTypes,
    nullptr
} };

void NoteGraphicsItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NoteGraphicsItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->formatColor(); break;
        case 1: _t->formatBold((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->formatAlign((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 3: _t->formatFontFamily((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->formatFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->currentCharFormatChanged((*reinterpret_cast<std::add_pointer_t<QTextCharFormat>>(_a[1]))); break;
        case 6: _t->cursorPositionChanged(); break;
        case 7: _t->insertImage(); break;
        case 8: _t->insertFormula(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *NoteGraphicsItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NoteGraphicsItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16NoteGraphicsItemE_t>.strings))
        return static_cast<void*>(this);
    return WidgetGraphicsItem::qt_metacast(_clname);
}

int NoteGraphicsItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WidgetGraphicsItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN16DataSourceWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto DataSourceWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16DataSourceWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DataSourceWidget",
        "dataSourceChanged",
        "",
        "objectSelected",
        "index",
        "propertySelected"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataSourceChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'objectSelected'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'propertySelected'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataSourceWidget, qt_meta_tag_ZN16DataSourceWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DataSourceWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DataSourceWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DataSourceWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DataSourceWidgetE_t>.metaTypes,
    nullptr
} };

void DataSourceWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataSourceWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataSourceChanged(); break;
        case 1: _t->objectSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->propertySelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DataSourceWidget::*)()>(_a, &DataSourceWidget::dataSourceChanged, 0))
            return;
    }
}

const QMetaObject *DataSourceWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DataSourceWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DataSourceWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DataSourceWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void DataSourceWidget::dataSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN16GraphMenuHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto GraphMenuHandler::qt_create_metaobjectdata<qt_meta_tag_ZN16GraphMenuHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GraphMenuHandler",
        "clearGraph",
        "",
        "configureGraph",
        "confApply",
        "QAbstractButton*",
        "button",
        "confChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'clearGraph'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'configureGraph'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'confApply'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'confChanged'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GraphMenuHandler, qt_meta_tag_ZN16GraphMenuHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GraphMenuHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemMenuHandler::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16GraphMenuHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16GraphMenuHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16GraphMenuHandlerE_t>.metaTypes,
    nullptr
} };

void GraphMenuHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GraphMenuHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clearGraph(); break;
        case 1: _t->configureGraph(); break;
        case 2: _t->confApply((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 3: _t->confChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *GraphMenuHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GraphMenuHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16GraphMenuHandlerE_t>.strings))
        return static_cast<void*>(this);
    return ItemMenuHandler::qt_metacast(_clname);
}

int GraphMenuHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemMenuHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN16MeterMenuHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto MeterMenuHandler::qt_create_metaobjectdata<qt_meta_tag_ZN16MeterMenuHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MeterMenuHandler",
        "configureMeter",
        "",
        "confApply",
        "QAbstractButton*",
        "button",
        "confChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'configureMeter'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'confApply'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'confChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MeterMenuHandler, qt_meta_tag_ZN16MeterMenuHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MeterMenuHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemMenuHandler::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MeterMenuHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MeterMenuHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16MeterMenuHandlerE_t>.metaTypes,
    nullptr
} };

void MeterMenuHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MeterMenuHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configureMeter(); break;
        case 1: _t->confApply((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 2: _t->confChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *MeterMenuHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MeterMenuHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MeterMenuHandlerE_t>.strings))
        return static_cast<void*>(this);
    return ItemMenuHandler::qt_metacast(_clname);
}

int MeterMenuHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemMenuHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN22ControllerGraphicsItemE_t {};
} // unnamed namespace

template <> constexpr inline auto ControllerGraphicsItem::qt_create_metaobjectdata<qt_meta_tag_ZN22ControllerGraphicsItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ControllerGraphicsItem",
        "incTriggered",
        "",
        "decTriggered",
        "sliderChanged",
        "value",
        "sliderReleased"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'incTriggered'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'decTriggered'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sliderChanged'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'sliderReleased'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ControllerGraphicsItem, qt_meta_tag_ZN22ControllerGraphicsItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ControllerGraphicsItem::staticMetaObject = { {
    QMetaObject::SuperData::link<WidgetGraphicsItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22ControllerGraphicsItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22ControllerGraphicsItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22ControllerGraphicsItemE_t>.metaTypes,
    nullptr
} };

void ControllerGraphicsItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ControllerGraphicsItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->incTriggered(); break;
        case 1: _t->decTriggered(); break;
        case 2: _t->sliderChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->sliderReleased(); break;
        default: ;
        }
    }
}

const QMetaObject *ControllerGraphicsItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ControllerGraphicsItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22ControllerGraphicsItemE_t>.strings))
        return static_cast<void*>(this);
    return WidgetGraphicsItem::qt_metacast(_clname);
}

int ControllerGraphicsItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WidgetGraphicsItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN21ControllerMenuHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto ControllerMenuHandler::qt_create_metaobjectdata<qt_meta_tag_ZN21ControllerMenuHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ControllerMenuHandler",
        "incTriggered",
        "",
        "decTriggered",
        "configureController",
        "confApply",
        "QAbstractButton*",
        "button",
        "confChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'incTriggered'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'decTriggered'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'configureController'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'confApply'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'confChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ControllerMenuHandler, qt_meta_tag_ZN21ControllerMenuHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ControllerMenuHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemMenuHandler::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ControllerMenuHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ControllerMenuHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21ControllerMenuHandlerE_t>.metaTypes,
    nullptr
} };

void ControllerMenuHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ControllerMenuHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->incTriggered(); break;
        case 1: _t->decTriggered(); break;
        case 2: _t->configureController(); break;
        case 3: _t->confApply((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 4: _t->confChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *ControllerMenuHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ControllerMenuHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ControllerMenuHandlerE_t>.strings))
        return static_cast<void*>(this);
    return ItemMenuHandler::qt_metacast(_clname);
}

int ControllerMenuHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemMenuHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN17TracerMenuHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto TracerMenuHandler::qt_create_metaobjectdata<qt_meta_tag_ZN17TracerMenuHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TracerMenuHandler",
        "clearTracer",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'clearTracer'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TracerMenuHandler, qt_meta_tag_ZN17TracerMenuHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TracerMenuHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemMenuHandler::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TracerMenuHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TracerMenuHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17TracerMenuHandlerE_t>.metaTypes,
    nullptr
} };

void TracerMenuHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TracerMenuHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clearTracer(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *TracerMenuHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TracerMenuHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TracerMenuHandlerE_t>.strings))
        return static_cast<void*>(this);
    return ItemMenuHandler::qt_metacast(_clname);
}

int TracerMenuHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemMenuHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
